// ====================================================================
//  Class:  DoubleJumpUT.DJ_InventoryItem
//  Parent: Botpack.TournamentPickup
//
//  This class is given to each player in the game. It intercepts the
//  jump command and does the stuff to allow for doublejumps and 
//  maximum compatibility with custom models.
//
//  (C)2003 by Mr.Mitchell
// ====================================================================

class DJ_InventoryItem extends TournamentPickup;

var int nofJumps;
var int maxJumps;
var int jumpHeight; 

var string keyNames[15]; // Support up to 15 keys that are bound to Jump
var int nofJumpKeys;

// Jumptype: 
// 0 = Jump at apex
// 1 = Jump while going up and at apex
// 2 = Jump always
var int  jumpType;

replication
{
	// Functions server can call on the client.
	reliable if( Role==ROLE_Authority )
		ClientPlayAnim;

	// Functions client can call on the server.
	reliable if( Role<ROLE_Authority )
		DoubleJump;
}

simulated function PreBeginPlay()
{
	local PlayerPawn P;

// Don't Super or the InventoryItem will be destroyed.
//	Super.PreBeginPlay();

	// The rest is only needed on a client
	if (Level.Netmode == NM_DedicatedServer) return;
	
	// Find the local playerpawn
	foreach AllActors(class 'PlayerPawn', P)
		if (Viewport(P.Player) != None)
			break;

	if (P == None) 
	{ 
		log("DJ: NO LOCAL PLAYERPAWN !!!");
		return;
	}
	
	// Initialize keys
	InitializeKeys(P);

	if (nofJumpKeys == 0)
	{
		// No keys were bound to Jump, this is a problem
		//log("DJ: Could NOT find any keys bound to JUMP/DOUBLEJUMP, DoubleJump will NOT work !!!");
	}
	else
	{
		// Explain how the user can restore his keys
		//log("DJ: Keys that were changed by DoubleJumpUT can be restored by typing \"restorejumpkeys\" in the UT console when DoubleJumpUT is running.");
	}
}


simulated function InitializeKeys(PlayerPawn P)
{
	local string keyName;
	local string keyBinding;
	local int i;
	
	nofJumpKeys = 0;

	// Figure out what keys are bound to Jump/DoubleJump
	for (i=0; i<255; i++)
	{
		keyName = P.ConsoleCommand("Keyname"@i);
		
		if ((InStr( P.ConsoleCommand("Keybinding"@keyName), "Jump") != -1) 
		     && (InStr( P.ConsoleCommand("Keybinding"@keyName), "DoubleJump") == -1))
		{
			// Key is bound to a Jump command and not DoubleJump, add DoubleJump in front
			keyBinding = P.ConsoleCommand("Keybinding"@keyName);
			P.ConsoleCommand("SET INPUT"@keyName@"DoubleJump|"$keyBinding);

			log("DJ: Changed"@keyName@"from"@keyBinding@"to DoubleJump|"$keyBinding);
			
			nofJumpKeys++;
		}
		else if (InStr( P.ConsoleCommand("Keybinding"@keyName), "DoubleJump") != -1)
		{
			// Found a key that was bound to DoubleJump
			nofJumpKeys++;
		}
	}
}

// Restore all keys that were bound to DoubleJump
exec simulated function RestoreJumpKeys()
{
	local int i, restoredKeys;
	local int FoundAt, StripStart;
	local string keyName;
	local string keyString, tempKeyString;
	local Actor P;

	log ("DJ: -- RestoreJumpKeys --");
	
	if (Owner == None)
	{
		return;
	}
	
	restoredKeys = 0;
	
	for (i=0; i<255; i++)
	{
		keyName = Owner.ConsoleCommand("Keyname"@i);
		keyString = Owner.ConsoleCommand("Keybinding"@keyName);
			
		// See if we added DoubleJump			
		if (InStr( keyString, "DoubleJump|") != -1)
		{
			FoundAt = InStr( KeyString, "DoubleJump|");
			StripStart = FoundAt + Len("DoubleJump|");
			tempKeyString = ""; 
			
			if (FoundAt != 0)
			{
				tempKeyString = Left(keyString, FoundAt);
			}

			Owner.ConsoleCommand("SET INPUT"@keyName@tempKeyString$Right(keyString, Len(keyString) - (FoundAt + Len("DoubleJump|"))));
			log("DJ: Changed key "$keyname$" from "$keyString$" to "$tempKeyString$Right(keyString, Len(keyString) - (FoundAt + Len("DoubleJump|"))));

			restoredKeys++;
		}
		else if (InStr( keyString, "DoubleJump |") != -1) // Maybe player fiddled around with his User.ini and added a space for the bar to make it look better
		{
			FoundAt = InStr( KeyString, "DoubleJump |");
			StripStart = FoundAt + Len("DoubleJump |");
			tempKeyString = ""; 
			
			if (FoundAt != 0)
			{
				tempKeyString = Left(keyString, FoundAt);
			}

			Owner.ConsoleCommand("SET INPUT"@keyName@tempKeyString$Right(keyString, Len(keyString) - (FoundAt + Len("DoubleJump |"))));
			log("DJ: Changed key "$keyname$" from "$keyString$" to "$tempKeyString$Right(keyString, Len(keyString) - (FoundAt + Len("DoubleJump |"))));

			restoredKeys++;
		}
	}
	
	if (restoredKeys == 0)
	{
		log("DJ: No keys could be restored.");
		PlayerPawn(Owner).ClientMessage("DoubleJumpUT: No keys could be restored.");
	}
	else
	{
		log("DJ: Jump keys are restored.");
		PlayerPawn(Owner).ClientMessage("DoubleJumpUT: Jump keys are restored.");
	}
}

exec function DoubleJump()
{
	local PlayerPawn P;

	if (PlayerPawn(Owner) == None) return;
	
	P = PlayerPawn(Owner);

	// Check for first jump, if we are falling it is NOT the first jump
	if (P.Physics == PHYS_Falling)
	{
		// Make sure we don't jump too much
		if (nofJumps < maxJumps)
		{
			// Second or later jump
			
			// Check if we are allowed to jump according to the JumpType
			switch (jumpType)
			{
				case 0: // Jump only at apex
					if (!(P.Physics == PHYS_Falling && Abs(P.Velocity.Z) < 100.0)) 
					{
						return;				
					}
					break;
				case 1: // Jump when going up and at apex
					if (!(P.Physics == PHYS_Falling && P.Velocity.Z > -100.0)) 
					{
						return;				
					}
					break;
			}
			
			// Jump
			P.Velocity.Z = P.JumpZ * jumpHeight;
			nofJumps++;

			// Do the jump sound
			P.PlaySound(P.JumpSound, SLOT_Talk, 1.5, true, 1200, 1.0 );
			if ( (Level.Game != None) && (Level.Game.Difficulty > 0) )
			{
				P.MakeNoise(0.1 * Level.Game.Difficulty);
			}

			// Do special stuff at last jump
			if (nofJumps == maxJumps)
			{
				// Play flip animation (Skaarj has roll animations)
				if ( P.AnimSequence=='DodgeR' && P.HasAnim('ROLLRIGHT') )
				{
					P.PlayAnim('RollRight', 1.35 * FMax(0.35, Region.Zone.ZoneGravity.Z/Region.Zone.Default.ZoneGravity.Z), 0.06);
					if (Level.Netmode != NM_StandAlone)
					{
						ClientPlayAnim('RollRight', 1.35 * FMax(0.35, Region.Zone.ZoneGravity.Z/Region.Zone.Default.ZoneGravity.Z), 0.06);
					}

				}
				else if ( P.AnimSequence=='DodgeL' && P.HasAnim('ROLLLEFT') )
				{
					P.PlayAnim('RollLeft', 1.35 * FMax(0.35, Region.Zone.ZoneGravity.Z/Region.Zone.Default.ZoneGravity.Z), 0.06);
					if (Level.Netmode != NM_StandAlone)
					{
						ClientPlayAnim('RollLeft', 1.35 * FMax(0.35, Region.Zone.ZoneGravity.Z/Region.Zone.Default.ZoneGravity.Z), 0.06);
					}

				} 
				else if ( P.HasAnim('Flip') )
				{
					P.PlayAnim('Flip', 1.35 * FMax(0.35, Region.Zone.ZoneGravity.Z/Region.Zone.Default.ZoneGravity.Z), 0.06);
					if (Level.Netmode != NM_StandAlone)
					{
						ClientPlayAnim('Flip', 1.35 * FMax(0.35, Region.Zone.ZoneGravity.Z/Region.Zone.Default.ZoneGravity.Z), 0.06);
					}
				}
			}
		}
	}
	else
	{
		// First jump, do the normal player jump
		P.DoJump();
		nofJumps = 1;
	}
}

simulated function ClientPlayAnim( name Sequence, optional float Rate, optional float TweenTime )
{
	Owner.PlayAnim(Sequence, Rate, TweenTime);
}


defaultproperties
{
    nofJumps=1
    ExpireMessage="ExpireMessage"
    PickupMessage="PickupMessage"
    ItemName="DoubleJumpUT"
    Mesh=LodMesh'Botpack.jboot'
    AmbientGlow=0
}
